/* An Arexx script for use with AmiSlate:  

  Mandelbrot generator for AmiSlate.

  Copyright (c) 1995,1996 Johan Torin  (johan@artworks.apana.org.au)

  Minor enhancements by Jeremy Friesner (jfriesne@ucsd.edu)
  
*/

parse arg CommandPort ActiveString

address (CommandPort)

if (length(CommandPort) == 0) then do
	say ""
	say "Usage:  rx mandelbrot.rexx <REXXPORTNAME>"
	say "        (REXXPORTNAME is usually AMISLATE)"
	say ""
	say "Or run from the Rexx menu within AmiSlate."
	say ""
	exit 0
	end

options results

/* Calculate size of drawing area */
GetWindowAttrs stem win.
X_Size = trunc((win.width-58))
Y_Size = trunc((win.height-53))
Num_Colors = 2 ** win.depth

/* Set colors, if the user wants */
EasyRequest "Mandelbrot" '"'||"Do you want to use the Mandelbrot palette?"||'"' "Yes|No"

/* Get object */
if (rc == 1) then do
	SetPenColor 0 0 0 0
	SetPenColor 1 0 0 15
	SetPenColor 2 2 2 15
	SetPenColor 3 4 4 15
	SetPenColor 4 6 6 15
	SetPenColor 5 8 8 15
	SetPenColor 6 10 10 15
	SetPenColor 7 12 12 15
	SetPenColor 8 14 14 15
	SetPenColor 9 15 15 15
	SetPenColor 10 15 15 10
	SetPenColor 11 15 15 5
	SetPenColor 12 15 15 0
	SetPenColor 13 15 13 0
	SetPenColor 14 15 12 0
	SetPenColor 15 15 11 0
	SetPenColor 16 15 9 0
	SetPenColor 17 15 8 0
	SetPenColor 18 15 7 0
	SetPenColor 19 15 6 0
	SetPenColor 20 15 4 0
	SetPenColor 21 15 3 0
	SetPenColor 22 15 2 0
	SetPenColor 23 15 0 0
	SetPenColor 24 13 0 2
	SetPenColor 25 12 0 4
	SetPenColor 26 10 0 6
	SetPenColor 27 9 0 8
	SetPenColor 28 7 0 10
	SetPenColor 29 6 0 12
	SetPenColor 30 4 0 14
	SetPenColor 31 2 0 15
	end

/* Init some variables */
PixelSize = 4                         /* The size of each dot in pixels */

/* Set this to SOLID, MOSAIC, TILE_BUBBLES, or OVERLAP_BUBBLES for different effects */
Effect = SOLID

/* Higher = slower & prettier, lower = faster */
Iterations = 32

/* Figure out which pen is black now, as it's expensive to do */
SetUserFColor 0 0 0
GetStateAttrs stem state.
blackpen = state.fpen

Z_r = 0
Z_i = 0
C_r = 0
C_i = 0
K_r = 0
K_i = 0
Delta_r = 0
Delta_i = 0
Itt = 0
X = 0
Y = 0

Start_r = -2.5
Start_i = -1.75
End_r = 1.5
End_i = 1.75

Ul_r = Start_r
Ul_i = Start_i
Lr_r = End_r
Lr_i = End_i

Delta_r = Lr_r-Ul_r
Delta_i = Lr_i-Ul_r

/* Since PixelSize is actually the height of the square... */
PixelSize = PixelSize - 1

Do Y = 0 to Y_Size-1 BY PixelSize+1
   C_i = Delta_i/Y_Size*Y+Ul_i
   Do X = 0 to X_Size-1 BY PixelSize+1
      C_r = Delta_r/X_Size*X+Ul_r
      Z_r = C_r
      Z_i = C_i
      K_r = Z_r * Z_r
      K_i = Z_i * Z_i
      Itt = 0
      Do While ((K_r + K_i) < 4) & (Itt < Iterations)
         Z_i = 2 * Z_r * Z_i + C_i
         Z_r = K_r - K_i + C_r
         K_r = Z_r * Z_r
         K_i = Z_i * Z_i
         Itt = Itt + 1
         end
      If Itt > Iterations-2 then Itt = -3     /* Kludge. */
      
      if (Effect == SOLID) then do
	      SetFPen Itt+4
	      Square X Y X+PixelSize Y+PixelSize FILL
	      end
      else if (Effect == MOSAIC) then do
	      SetFPen Itt+4
	      Square X Y X+PixelSize Y+PixelSize FILL
	      SetFPen BlackPen
	      Square X Y X+PixelSize Y+PixelSize 
	      end	      	   	      
      else if (Effect == OVERLAP_BUBBLES) then do
	      SetFPen Itt+4
	      Circle X Y PixelSize PixelSize FILL
	      SetFPen BlackPen
	      Circle X Y PixelSize PixelSize
	      end
      else if (Effect == TILE_BUBBLES) then do
	      SetFPen Itt+4
	      Circle X Y PixelSize%2 PixelSize%2 FILL
	      SetFPen BlackPen
	      Circle X Y PixelSize%2 PixelSize%2
	      end
   End
End
